# -*- coding: utf-8 -*-
#
# Copyright (c) 2014 Rodolphe Quiédeville <rodolphe@quiedeville.org>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.

from jinja2 import Environment, FileSystemLoader
from django_aggtrigg.util import parse_where_clause
import os


class TriggerPostgreSQL(object):

    def __init__(self):
        self.name = "PostgreSQL"

    def sql_create_table(self, name, column, aggregats):
        """Return a SQL statement to create a FUNCTION

        name : functions name
        column (tuple): column name and typname
        aggregats (array): aggregats used

        - Each aggregate can be of 2 kind. string or dict.

        string is the simplest it can be on of count, max, sum, etc...

        dict is in the form:
            - aggregate_type:
                - pretty name
                    - list of filters with field and value.
        example::

            {"count":
                [
                    {"private": [
                        {"field": "is_private", "value": True}
                       ]
                    }
                ]
            }

        each "agg_type" can have more than one aggregate (one for
        public another for private for example), each aggregate can
        have multiple filters. All filters are ANDed OR and NOR is not
        possible for the moment.

        """
        tpl = os.path.join(os.path.dirname(__file__), 'templates')
        env = Environment(loader=FileSystemLoader(tpl))

        template = env.get_template('pg_create_table.sql')
        aggs = []

        for agg in aggregats:
            col_type = column[1]
            if agg == "count":
                col_type = "integer"
            if isinstance(agg, dict):
                agg_type = agg.keys()[0]
                if agg_type == "count":
                    col_type = "integer"
                for trigger in agg[agg_type]:
                    agg_name = trigger.keys()[0]
                    aggs.append(
                        {"name": "{}_{}".format(agg_type,
                                                agg_name),
                         "type": col_type})
            else:
                aggs.append({"name": agg,
                             "type": col_type})
        temp = template.render(
            name=name,
            column={"name": column[0], "type": column[1]},
            aggregats=aggs)
        return temp

    def sql_drop_table(self, name):
        """Return a command to drop a table in PostgreSQL

        name (string) : the table name
        """
        return "DROP TABLE IF EXISTS {0}".format(name)

    def sql_drop_trigger(self, name, table):
        """Return a command to drop a trigger in PostgreSQL

        name (string): the triggers's name
        table (string) : the table name
        """
        return "DROP TRIGGER IF EXISTS {0} ON {1}".format(name, table)

    def sql_drop_function(self, name):
        """Return a command to drop a function in PostgreSQL

        name (string): the function's name to drop
        """
        return "DROP FUNCTION IF EXISTS {0}".format(name)

    def sql_create_trigger(self, name, function, table, action):
        """Return the SQL statement to create a trigger

        name (string): the function's name to call
        """
        response = """CREATE TRIGGER {0} AFTER {3} ON {2}
        FOR EACH ROW
        EXECUTE PROCEDURE {1}""".format(name, function, table,
                                        action.upper())
        return response

    def sql_create_function_insert(self, name, table,
                                   column, aggtable, action,
                                   where_clause=None,
                                   agg_key=None):
        """Return a SQL statement to create a FUNCTION

        name : function name
        table (string) : the originate of datas
        column : column name
        aggtable : the table where data will be stored
        """
        tpl = os.path.join(os.path.dirname(__file__), 'templates')
        env = Environment(loader=FileSystemLoader(tpl))
        template = env.get_template('pg_function_insert.sql')
        if agg_key is None:
            action_key = "agg_count"
            actions = "agg_count=agg_count+1"
        else:
            action_key = "agg_count_{0}".format(agg_key)
        actions = "{0}={0}+1".format(action_key)
        insert_values = 1
        if where_clause:
            where_clause = parse_where_clause(where_clause, table, "NEW")
        tmp = template.render(name=name,
                              table=table,
                              aggtable=aggtable,
                              column=column,
                              actions=actions,
                              action_key=action_key,
                              insert_values=insert_values,
                              where_clause=where_clause)
        return tmp

    def sql_create_function_delete(self, name, table,
                                   column, aggtable, action,
                                   where_clause=None,
                                   agg_key=None):
        """Return a SQL statement to create a FUNCTION

        name : functions name
        column : column name
        aggtable : the table where data will be stored
        action : action that fired the trigger
        """
        tpl = os.path.join(os.path.dirname(__file__), 'templates')
        env = Environment(loader=FileSystemLoader(tpl))
        template = env.get_template('pg_function_delete.sql')

        if agg_key is None:
            actions = "agg_count=agg_count-1"
        else:
            actions = "agg_count_{0}=agg_count_{0}-1".format(agg_key)

        if where_clause:
            where_clause = parse_where_clause(where_clause, table, "OLD")

        tmp = template.render(name=name,
                              table=aggtable,
                              column=column,
                              actions=actions,
                              where_clause=where_clause)
        return tmp

    def sql_create_function_update(self, name, table,
                                   column, aggtable, action,
                                   where_clause=None,
                                   agg_key=None):
        """Return a SQL statement to create a FUNCTION

        name : functions name
        column : column name
        aggtable : the table where data will be stored
        action : action that fired the trigger
        """
        tpl = os.path.join(os.path.dirname(__file__), 'templates')
        env = Environment(loader=FileSystemLoader(tpl))

        template = env.get_template('pg_function_update.sql')
        if agg_key is None:
            action_key = "agg_count"
        else:
            action_key = "agg_count_{0}".format(agg_key)
        actions_new = "{0}={0}+1".format(action_key)
        actions_old = "{0}={0}-1".format(action_key)

        if where_clause:
            old_where_clause = parse_where_clause(where_clause, table, "OLD")
            where_clause = parse_where_clause(where_clause, table, "NEW")
        else:
            old_where_clause = None

        temp = template.render(name=name,
                               table=aggtable,
                               column=column,
                               actions_old=actions_old,
                               actions_new=actions_new,
                               action_key=action_key,
                               where_clause=where_clause,
                               old_where_clause=old_where_clause)
        return temp

    def sql_get_column_typname(self):
        """Return the query to find the typename of a column
        """
        qry = " ".join(["SELECT typname",
                        "FROM pg_class,pg_attribute,pg_type",
                        "WHERE relname=%s",
                        "AND attname=%s",
                        "AND atttypid=pg_type.oid",
                        "AND attrelid=pg_class.oid"])
        return (qry, None)

    def sql_init_aggtable(self, name, column, aggname, aggregats):
        """Return a SQL statement to initialize aggregate table

        name : functions name
        column (tuple): column name and typname
        aggname (string) : aggregate table name
        aggregats (array): aggregats used
        """

        tpl = os.path.join(os.path.dirname(__file__), 'templates')
        env = Environment(loader=FileSystemLoader(tpl))

        template = env.get_template('pg_init.sql')

        temp = template.render(table=name,
                               aggtable=aggname,
                               column=column,
                               aggregats=aggregats)
        return temp

    def sql_nb_tuples(self):
        """Return a SQL statement to know appox number of tuples in a table
        """
        qry = """SELECT reltuples FROM pg_class WHERE relname=%s"""
        return qry

    def agg_fname(self, agg, fname):

        AGGS = {'count': 'SUM',
                'max': 'GREATEST',
                'min': 'LEAST'}

        try:
            res = "%s(%s)" % (AGGS[agg], fname)
        except KeyError:
            res = fname

        return res

    def sql_table_exists(self):
        """Return the query to find the typename of a column
        """
        qry = " ".join(["SELECT count(relname)",
                        "FROM pg_class",
                        "WHERE relname=%s",
                        "AND relkind='r'"])
        return qry

    def sql_trigger_on_table_exists(self):
        """Return the query to find the typename of a column
        """
        qry = " ".join(["SELECT count(pg_trigger.tgname)",
                        "FROM pg_trigger,pg_class",
                        "WHERE pg_trigger.tgrelid=pg_class.oid",
                        "AND pg_trigger.tgname=%s",
                        "AND pg_class.relname=%s"])
        return qry

    def sql_trigger_function_on_table_exists(self):
        """Return the query to find the typename of a column
        """
        qry = " ".join(["SELECT count(pg_trigger.tgname)",
                        "FROM pg_trigger,pg_class,pg_proc",
                        "WHERE pg_trigger.tgrelid=pg_class.oid",
                        "AND pg_trigger.tgfoid=pg_proc.oid",
                        "AND pg_proc.proname=%s",
                        "AND pg_class.relname=%s"])
        return qry



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2014 Rodolphe Quiédeville <rodolphe@quiedeville.org>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of django-aggtrigg nor the names of its contributors
#   may be used to endorse or promote products derived from this software
#   without specific prior written permission.
#
import sys
from django.core.management.base import BaseCommand
from django.conf import settings
from optparse import make_option
from ... import util
from .. import djutil


class Command(BaseCommand):
    help = 'Import datas'
    option_list = BaseCommand.option_list + (
        make_option("-s",
                    "--simulate",
                    dest="simulate",
                    action="store_true",
                    help="juste print on stdout SQL commande, do nothing",
                    default=False),
        make_option("-t",
                    "--table",
                    dest="table",
                    type="string",
                    help="table name",
                    default=None),
        make_option("-c",
                    "--column",
                    dest="column",
                    type="string",
                    help="column name",
                    default=None),
        make_option("-d",
                    "--database",
                    dest="database",
                    type="string",
                    help="table name",
                    default="default"),
        make_option("-q",
                    "--quiet",
                    dest="quiet",
                    action="store_true",
                    default=False))

    def handle(self, *args, **options):
        """
        Handle action
        """
        if options['quiet']:
            options['verbosity'] = 0

        for trig in djutil.get_agg_fields():
            self.create_trigger(trig, options)

    def create_trigger(self, trig, options):
        """
        {'table': u'apple_apple',
         'model': <class 'foo.apple.models.Apple'>,
         'aggs': ['max'],
         'field': 'indice'}
        """
        aggs = trig['aggs']
        # rely on django to find the "real" column name. foreignkeys
        # names are "relation" but column is "relation_id"
        model = trig["model"]
        column = model._meta.get_field(trig['field']).attname
        table = trig['table']
        engine = settings.DATABASES[options['database']]['ENGINE']
        agg = util.AggTrigger(engine, table, column, aggs, model=model)
        agg.verbose = int(options['verbosity'])

        comment = "-- table:    %s\n-- column:   %s\n-- aggregat: %s\n"

        if table and column and len(aggs) > 0:
            if not options['simulate']:
                agg.create_objects()
                if options['verbosity'] > 0:
                    sys.stdout.write(comment % (table, column, aggs))
                    sys.stdout.write("Create table : %s\n" % (agg.table_name))

            #  do nothing
            else:
                sys.stdout.write("%s\n" % (agg.sql_create_table()))
                for sql in agg.sql_create_functions():
                    sys.stdout.write(comment % (table, column, aggs))
                    sys.stdout.write("%s\n" % (sql))
                for tgs in agg.sql_create_triggers():
                    sys.stdout.write("%s\n" % (tgs[1]))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2014 Rodolphe Quiédeville <rodolphe@quiedeville.org>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of django-aggtrigg nor the names of its contributors
#   may be used to endorse or promote products derived from this software
#   without specific prior written permission.
#
import sys
from optparse import make_option
from django.core.management.base import BaseCommand
from django.conf import settings
from ... import util
from .. import djutil


class Command(BaseCommand):
    """Print SQL Statements on STDOUT to DROP all objects
    """
    help = 'Drop database relations'
    option_list = BaseCommand.option_list + (
        make_option("-t",
                    "--table",
                    dest="table",
                    type="string",
                    help="table name",
                    default=None),
        make_option("-c",
                    "--column",
                    dest="column",
                    type="string",
                    help="column name",
                    default=None),
        make_option("-d",
                    "--database",
                    dest="database",
                    type="string",
                    help="table name",
                    default="default"))

    def handle(self, *args, **options):
        """Do the job
        """
        for trig in djutil.get_agg_fields():
            self.drop_trigger(trig, options)

    def drop_trigger(self, trig, options):
        aggs = trig['aggs']
        model = trig["model"]
        column = model._meta.get_field(trig['field']).attname
        table = trig['table']

        engine = settings.DATABASES[options['database']]['ENGINE']

        agg = util.AggTrigger(engine, table, column, aggs, model=model)

        for sqltrig in agg.sql_drop_triggers():
            sys.stdout.write("%s;\n" % (sqltrig))

        for sqlfunc in agg.sql_drop_functions():
            sys.stdout.write("%s;\n" % (sqlfunc))
        sys.stdout.write("%s;\n" % (agg.sql_drop_table()))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2014 Rodolphe Quiédeville <rodolphe@quiedeville.org>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of django-aggtrigg nor the names of its contributors
#   may be used to endorse or promote products derived from this software
#   without specific prior written permission.
#
import sys
from optparse import make_option
from django.core.management.base import BaseCommand
from django.conf import settings
from ... import util
from .. import djutil


class Command(BaseCommand):
    help = 'Import datas'
    option_list = BaseCommand.option_list + (
        make_option("-d",
                    "--database",
                    dest="database",
                    type="string",
                    help="table name",
                    default="default"),
        make_option("-q",
                    "--quiet",
                    dest="quiet",
                    action="store_true",
                    default=False)
    )

    def handle(self, *args, **options):
        """
        Read the table book without TextField
        """
        for trig in djutil.get_agg_fields():
            self.init_agg(trig, options)

    def init_agg(self, trig, options):
        """
        {'table': u'apple_apple',
         'model': <class 'foo.apple.models.Apple'>,
         'aggs': ['max'],
         'field': 'indice'}
        """
        engine = settings.DATABASES[options['database']]['ENGINE']

        agg = util.AggTrigger(
            engine,
            trig['table'],
            trig["model"]._meta.get_field(trig['field']).attname,
            trig['aggs'],
            model=trig["model"])

        agg.verbose = int(options['verbosity'])

        message = u" ".join([u"Are you sure you want to initialize %s ?\n",
                             u"%s contains %s tuples approximatively,",
                             u"maybe long : [y/N] (please type yes to do)\n"])

        if options.get("quiet"):
            answer = "yes"
        else:
            if sys.version_info.major == 2:
                input = raw_input  # noqa
            answer = input(message % (agg.table_name,
                                      trig['table'],
                                      agg.get_nb_tuples()))
        if answer == "yes":
            sys.stdout.write(" Initialize %s ...\n" % (agg.table_name))
            agg.initialize()
            sys.stdout.write(" done, %s is initialized\n" % (agg.table_name))
        else:
            return



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014 Rodolphe Quiédeville <rodolphe@quiedeville.org>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of django-aggtrigg nor the names of its contributors
#   may be used to endorse or promote products derived from this software
#   without specific prior written permission.
#
from django.db import models
from django.db import connection
from djqmixin import QMixin
import tool
import util


class TooManyKwargs(Exception):
    pass


class MissingKwargs(Exception):
    pass


class TriggerFieldMixin(object):
    def __init__(self, *args, **kwargs):
        self.aggregate_trigger = ['count']
        super(TriggerFieldMixin, self).__init__(*args, **kwargs)


class ForeignKeyTriggerField(TriggerFieldMixin, models.ForeignKey):

    description = "An ForeignKeyField with trigger"


class IntegerTriggerField(TriggerFieldMixin, models.IntegerField):

    description = "An IntegerField with trigger"


class FloatTriggerField(TriggerFieldMixin, models.FloatField):

    description = "An FloatField with trigger"


class AggCount(QMixin):

    def get_count(self):
        """Returns a new QuerySet object.  Subclasses can override this method
        to easily customize the behavior of the Manager.
        """
        qs = self
        # Introspection part, we want to know if some of the reverse
        # relations of the models contains some TriggerFieldMixin
        # pointing to the model we are working with.

        for field in self.model.__dict__.itervalues():
            if isinstance(
                    field,
                    models.fields.related.ForeignRelatedObjectsDescriptor):

                # the model we are working with has a reverse relation

                if isinstance(field.related.field, ForeignKeyTriggerField):
                    # ForeignKeyTriggerField is the type of relation
                    # we are looking for. We need to retreive the
                    # table where the aggregates are lying

                    table = "{}__{}_agg".format(
                        field.related.model._meta.db_table,
                        field.related.field.attname)
                    # Now that we have the table, we need to retreive
                    # the revelant fields name on this table to create
                    # the extra(s) we need.
                    #
                    # There is 2 forms of aggregate.
                    #
                    # The simplest is a string: "count", "sum", "max"...
                    #
                    # The second is a dict we only need to digg into
                    # the dict to retreive the type of aggregate
                    # (count, max, sum etc...) and for each type, the
                    # list of aggregate name
                    for agg in field.related.field.aggregate_trigger:
                        select = {}
                        filters = []
                        if isinstance(agg, dict):
                            # {"count":
                            #      [{"private":
                            #           {"field":
                            #              "somefield",
                            #            "filters":
                            #               "somefilters"},
                            #      ...]
                            for aggregate_type in agg.iterkeys():
                                for agg_filter in agg[aggregate_type]:
                                    for name in agg_filter:
                                        filters.append(
                                            "{}_{}".format(aggregate_type,
                                                           name))
                        else:
                            filters = [agg]

                        for filter in filters:
                            # We have to construct the query to call
                            # in the extra, a little more
                            # introspection here
                            #
                            # - "agg_{}".format(filter) => the filter we
                            #                              found earlier, with
                            #                              a agg_ prepended to
                            #                              avoid name clash
                            #
                            # - table : the table name found earlier
                            #
                            # - field.related.field.attname => the column name
                            #                          of the relation to the
                            #                          working models on the
                            #                          reverse related model
                            #                          (are you still with me?)
                            #                          Working model is Article
                            #                          ArticleComment has a
                            #                          foreignkey to Article,
                            #                          named "article", in the
                            #                          database, the column is
                            #                          article_id
                            # - self.model._meta.db_table => the model table we
                            #                                are working on
                            # - self.model._meta.pk.name => the column name of
                            #                               the working model
                            #                               pk
                            #
                            param = "select {} from {} where {}={}.{}".format(
                                "agg_{}".format(filter),
                                table,
                                field.related.field.attname,
                                self.model._meta.db_table,
                                self.model._meta.pk.name)
                            # create the select and create an extra for it
                            select["{}_{}".format(
                                field.related.var_name, filter)] = param
                            qs = qs.extra(select=select)
        # finaly return the augmented queryset
        return qs


class AggTriggManager(models.Manager):

    def optimized_count(self, **kwargs):
        """Return count in optimized manner

        Apple.objects.filter(indice=4).count()
        Apple.objects.optimized_count(indice=4)

        """
        if (len(kwargs) == 0):
            raise MissingKwargs("you must specify the filter clause")
        if (len(kwargs) > 1):
            raise TooManyKwargs()
        # do not care about more than one kwarg
        (key, value) = kwargs.items()[0]
        (nbe, column, qfilter) = tool.parse_kwarg(key)

        agg = 'agg_count'
        agf = agg.split('_')

        agt = util.AggTrigger(util.PG_BACKEND, self.model._meta.db_table,
                              column)

        if nbe == 1:
            agf = agg
        elif nbe == 2:
            agf = agt.agg_function(agg, agf[1])

        cursor = connection.cursor()
        qry = """SELECT {} FROM {} WHERE {} %s"""

        tbname = util.table_name(self.model._meta.db_table, column)
        cursor.execute(qry.format(agf, tbname, qfilter), [value])
        row = cursor.fetchone()
        return row[0]


# As the fields define here are not special types, it's easy to let
# south add them to a migration. The code below will make the revelant
# work to make south able to manage the migration.
# cf. http://south.aeracode.org/wiki/MyFieldsDontWork for the explanation
try:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules(
        [], ["^django_aggtrigg\.models\.IntegerTriggerField"])
    add_introspection_rules(
        [], ["^django_aggtrigg\.models\.FloatTriggerField"])
except ImportError:
    pass



#! /usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (C) 2012 Rodolphe Quiedeville <rodolphe@quiedeville.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest
from pg import TriggerPostgreSQL


class utilTests(unittest.TestCase):

    def setUp(self):
        self.agg = TriggerPostgreSQL()
        self.maxDiff = None

    def test_init(self):
        trig = TriggerPostgreSQL()
        self.assertEqual('PostgreSQL', trig.name)

    def test_sql_drop_trigger(self):
        res = self.agg.sql_drop_trigger('foo', 'table_bar')
        self.assertTrue(isinstance(res, str))
        #  TODO mov test to pgTests
        #  self.assertEqual(res, 'DROP TRIGGER IF EXISTS foo ON table_bar')

    def test_sql_drop_function(self):
        res = self.agg.sql_drop_function('foo')
        self.assertTrue(isinstance(res, str))

    def test_sql_drop_table(self):
        res = self.agg.sql_drop_table('table_bar')
        self.assertTrue(isinstance(res, str))

    def test_sql_create_function_insert(self):
        trig = TriggerPostgreSQL()
        res = trig.sql_create_function_insert('book_nbpage_insert()',
                                              'book',
                                              'nbpage',
                                              'book_nbpage')

        attend = u"""CREATE OR REPLACE FUNCTION book_nbpage_insert() RETURNS TRIGGER AS $BODY$
BEGIN
IF (SELECT nbpage FROM book_nbpage WHERE nbpage=NEW.nbpage) IS NOT NULL THEN
    UPDATE book_nbpage SET agg_count=agg_count+1 WHERE nbpage=NEW.nbpage;
ELSE
    INSERT INTO book_nbpage VALUES (NEW.nbpage, 1);
END IF;
RETURN NEW;
END;
$BODY$ LANGUAGE plpgsql;"""
        self.assertEqual(res, attend)

    def test_sql_create_table(self):
        """Test with only one aggregat
        """
        res = self.agg.sql_create_table('book', 'nbpage', ['count'])

        attend = u"""CREATE TABLE book (nbpage int4, agg_count int4);
CREATE INDEX ON book (nbpage);"""

        self.assertEqual(res, attend)

    def test_sql_create_table_twoagg(self):
        """Test with two aggregats
        """
        res = self.agg.sql_create_table('book', 'nbpage', ['count', 'min'])

        attend = u"""CREATE TABLE book (nbpage int4, agg_count int4, agg_min int4);
CREATE INDEX ON book (nbpage);"""

        self.assertEqual(res, attend)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014 Rodolphe Quiédeville <rodolphe@quiedeville.org>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of django-aggtrigg nor the names of its contributors
#   may be used to endorse or promote products derived from this software
#   without specific prior written permission.
#
from django.db.models import get_models
from django_aggtrigg.models import TriggerFieldMixin


def get_agg_fields():
    """Return all special field defined
    """
    trigg = []

    for model in get_models():

        for field in model._meta.fields:

            if isinstance(field, TriggerFieldMixin):
                trigg.append({"model": model,
                              "table": model._meta.db_table,
                              "field": field.name,
                              "aggs": field.aggregate_trigger})

    return trigg



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2014 Rodolphe Quiédeville <rodolphe@quiedeville.org>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of django-aggtrigg nor the names of its contributors
#   may be used to endorse or promote products derived from this software
#   without specific prior written permission.
#
import sys
from django.core.management.base import BaseCommand
from django.conf import settings
from optparse import make_option
from ... import util
from .. import djutil

STATUS = ['OK', 'KO']


class Command(BaseCommand):
    """Check database coherence with models
    """
    help = 'Import datas'
    option_list = BaseCommand.option_list + (
        make_option("-d",
                    "--database",
                    dest="database",
                    type="string",
                    help="database name",
                    default="default"),)

    def handle(self, *args, **options):
        """Handle action
        """
        trigs = djutil.get_agg_fields()
        sys.stdout.write("--found %d triggers\n" % (len(trigs)))
        for trig in trigs:
            self.check_trigger(trig, options)

    def check_trigger(self, trig, options):
        """Check database for a trigger

        trig : {'table': u'apple_apple',
        'model': <class 'foo.apple.models.Apple'>,
         'aggs': ['max'],
         'field': 'indice'}
        """
        aggs = trig['aggs']
        table = trig['table']
        model = trig["model"]
        column = model._meta.get_field(trig['field']).attname
        engine = settings.DATABASES[options['database']]['ENGINE']

        agg = util.AggTrigger(engine, table, column,
                              aggs, model=model)

        agg.verbose = int(options['verbosity'])

        comment = "\n".join(["--",
                             "-- model: %s",
                             "-- source: %s, column: %s, aggregats: %s"])

        if table and trig['field'] and len(aggs) > 0:
            sys.stdout.write(comment % (trig['model'], table,
                                        trig['field'], aggs))
            if agg.agg_table_ispresent():
                msg = "OK table: %s is present\n" % (agg.table_name)
            else:
                msg = "KO table: %s is absent\n" % (agg.table_name)

            sys.stdout.write(msg)

            for trig in agg.triggers_on_table_are_present():
                if trig[1]:
                    msg = "OK trigger: %s is present\n" % (trig[0])
                else:
                    msg = "KO trigger: %s is absent\n" % (trig[0])
                sys.stdout.write(msg)

            for func in agg.functions_are_present():
                if func[1]:
                    msg = "OK function: %s is present\n" % (func[0])
                else:
                    msg = "KO function: %s is absent\n" % (func[0])
                sys.stdout.write(msg)
